#for AISVision_Predict_API_1.6.1
import ctypes
from ctypes import c_void_p, c_char, c_char_p, c_bool, c_int, c_float, Structure, POINTER
import os
import numpy

# AISVision用DLLのロード
os.add_dll_directory(os.path.join(os.path.dirname(__file__)) )
aisv_predictor = ctypes.CDLL("libxditk_aisv_predictor.dll")
os.add_dll_directory(os.path.join(os.environ['CUDA_PATH_V11_8'], 'bin'))

# 列挙型の定義
class ImageDepth(ctypes.c_int):
    """Bit深度
    Attributes:
        IMAGE_DEPTH_UINT8 (0): 符号なし8bit
        IMAGE_DEPTH_UINT16 (1): 符号なし16bit
        NUM_IMAGE_DEPTH (2): 符号なし8bit
    """
    IMAGE_DEPTH_UINT8 = 0
    IMAGE_DEPTH_UINT16 = 1
    NUM_IMAGE_DEPTH = 2
    name = {
        0: "IMAGE_DEPTH_UINT8",
        1: "IMAGE_DEPTH_UINT16",
        2: "NUM_IMAGE_DEPTH"
    }

class ModelType(ctypes.c_int):
    """モデルタイプ
    Attributes:
        NONE (0): なし
        SEGMENTATION (1): セグメンテーション
        OBJECTDETECTION (2): 物体検知
        CLASSIFICATION (3): クラス分類
        ANOMALYDETECTION (4): 異常検知
        ORIENTEDOBJECTDETECTION (5): 角度付き物体検知
    """
    NONE = 0
    SEGMENTATION = 1
    OBJECTDETECTION = 2
    CLASSIFICATION = 3
    ANOMALYDETECTION = 4
    ORIENTEDOBJECTDETECTION = 5
    name = {
        0: "NONE",
        1: "SEGMENTATION",
        2: "OBJECTDETECTION",
        3: "CLASSIFICATION",
        4: "ANOMALYDETECTION",
        5: "ORIENTEDOBJECTDETECTION"
    }

class ModeType(ctypes.c_int):
    """モードタイプ
    Attributes:
        NORMAL (0): Normal(通常)
        FAST (1): Fast(高速)
        ENHANCED (2): Enhanced(拡張)
    """
    NORMAL = 0
    FAST = 1
    ENHANCED = 2
    name = {
        0: "NORMAL",
        1: "FAST",
        2: "ENHANCED"
    }

class AisvStatus(ctypes.c_int):
    """実行ステータス
    Attributes:
        AISV_SUCCESS (0): 処理が成功しました。
        AISV_LOADEDSUCCESSFULLY_VERSION_LESSTHAN_CURVERSION (1): モデルのバージョンが現在のDLLのバージョンより古いです。
        AISV_NOTLOADED (2): モデルがロードされていません。
        AISV_DONGLE_ERROR (1001): ドングルチェックが失敗しました。ドングルを確認してください。
        AISV_PATHNOTEXIST (1002): 入力されたパスが存在しません。
        AISV_EMPTYDATA (1003): 入力データが空です。
        AISV_LOADFAIL_NOTSUPPORTEDVERSION (2001): 入力されたモデルのバージョンはサポートされていません。
        AISV_LOADFAIL_DECRYPTERROR (2002): モデルの復号化に失敗しました。モデルのバージョンを確認してください。
        AISV_LOADFAIL_NOTSUPPORTEDMODELTYPE (2003): このモデルタイプはこのDLLでサポートされていません。モデルを確認してください。
        AISV_LOADFAIL_GPUERROR (2004): GPUデバイスエラーが発生しました。
        AISV_LOADFAIL_ONLYALLOWGPU (2005): 現在のモデルタイプは use_gpu = true の場合のみ実行可能です。
        AISV_LOADFAIL_XMLINITERROR (5001): 初期化に失敗しました。plugins.xmlの存在とDLLの依存関係を確認してください。
        AISV_LOADFAIL_IGPUNOTFOUND (5002): iGPUが見つかりません。
        AISV_UNKNOWN (9000): 不明なエラーが発生しました。
    """
    AISV_SUCCESS = 0
    AISV_LOADEDSUCCESSFULLY_VERSION_LESSTHAN_CURVERSION = 1
    AISV_NOTLOADED = 2
    AISV_DONGLE_ERROR = 1001
    AISV_PATHNOTEXIST = 1002
    AISV_EMPTYDATA = 1003
    AISV_LOADFAIL_NOTSUPPORTEDVERSION = 2001
    AISV_LOADFAIL_DECRYPTERROR = 2002
    AISV_LOADFAIL_NOTSUPPORTEDMODELTYPE = 2003
    AISV_LOADFAIL_GPUERROR = 2004
    AISV_LOADFAIL_ONLYALLOWGPU = 2005
    AISV_LOADFAIL_XMLINITERROR = 5001
    AISV_LOADFAIL_IGPUNOTFOUND = 5002
    AISV_UNKNOWN = 9000
    name = {
        0: "AISV_SUCCESS",
        1: "AISV_LOADEDSUCCESSFULLY_VERSION_LESSTHAN_CURVERSION",
        2: "AISV_NOTLOADED",
        1001: "AISV_DONGLE_ERROR",
        1002: "AISV_PATHNOTEXIST",
        1003: "AISV_EMPTYDATA",
        2001: "AISV_LOADFAIL_NOTSUPPORTEDVERSION",
        2002: "AISV_LOADFAIL_DECRYPTERROR",
        2003: "AISV_LOADFAIL_NOTSUPPORTEDMODELTYPE",
        2004: "AISV_LOADFAIL_GPUERROR",
        2005: "AISV_LOADFAIL_ONLYALLOWGPU",
        5001: "AISV_LOADFAIL_XMLINITERROR",
        5002: "AISV_LOADFAIL_IGPUNOTFOUND",
        9000: "AISV_UNKNOWN"
    }

# 構造体の定義
class AisvColor(Structure):
    """AisvColorオブジェクト
    Attributes:
        r (int): 赤の輝度値
        g (int): 緑の輝度値
        b (int): 青の輝度値
    """
    _fields_ = [
        ("r", c_int),
        ("g", c_int),
        ("b", c_int)
    ]

class AisvImage(Structure):
    """AisvImageバッファ
    Attributes:
        width (int): 横解像度
        height (int): 縦解像度
        channel (int): カラーチャンネル
        depth (ImageDepth): ImageDepthオブジェクト
        data (void): RGB並びで格納された画像データの先頭アドレス
    """
    _fields_ = [
        ("width", c_int),
        ("height", c_int),
        ("channel", c_int),
        ("depth", ImageDepth),
        ("data", c_void_p)
    ]

class AisvCategoryItem(Structure):
    """AisvCategoryItemオブジェクト
    Attributes:
        index (int): カテゴリ番号
        name (string): カテゴリ名
        color (AisvColor): AisvColorオブジェクト
        threshold (float): 閾値
    """
    _fields_ = [
        ("index", c_int),
        ("name", c_char * 32),
        ("color", AisvColor),
        ("threshold", c_float)
    ]

class AisvCategoryInfo(Structure):
    """AisvCategoryInfoオブジェクト
    Attributes:
        category_list (AisvCategoryItem): AisvCategoryItemオブジェクト
        size (int): リストサイズ
    """
    _fields_ = [
        ("category_list", AisvCategoryItem * 256),
        ("size", c_int)
    ]

class AisvModelInfo(Structure):
    """AisvModelInfoオブジェクト
    Attributes:
        model_type (ModelType): ModelTypeオブジェクト
        model_status (AisvStatus): AisvStatusrオブジェクト
        project_name (string): プロジェクト名
        task_name (string): タスク名
        project_version (string): プロジェクトバージョン
        mode_type (ModeType): ModeTypeオブジェクト
        extra (void): 未定義(拡張用)
    """
    _fields_ = [
        ("model_type", ModelType),
        ("model_status", AisvStatus),
        ("project_name", c_char * 32),
        ("task_name", c_char * 32),
        ("project_version", c_char * 32),
        ("mode_type", ModeType),
        ("extra", c_void_p)
    ]

class AnomalyDetectionInfo(Structure):
    """AnomalyDetectionInfoオブジェクト
    Attributes:
        class_id (int): カテゴリ番号
        mask (AisvImage): AisvImageオブジェクト
        color (AisvColor): AisvColorオブジェクト
        name (string): カテゴリ名
        score (float): 可能性(1 = 100%, 0 = 0%)
    """
    _fields_ = [
        ("class_id", c_int),
        ("mask", AisvImage),
        ("color", AisvColor),
        ("name", c_char * 32),
        ("score", c_float)
    ]

class ClassificationInfo(Structure):
    """ClassificationInfoオブジェクト
    Attributes:
        class_id (int): カテゴリ番号
        prob (float): 可能性(1 = 100%, 0 = 0%)
        color (AisvColor): AisvColorオブジェクト
        name (string): カテゴリ名
    """
    _fields_ = [
        ("class_id", c_int),
        ("prob", c_float),
        ("color", AisvColor),
        ("name", c_char * 32)
    ]

class ObjectDetectionItem(Structure):
    """ObjectDetectionItemオブジェクト
    Attributes:
        prob (float): 可能性(1 = 100%, 0 = 0%)
        width (float): 横解像度
        height (float): 縦解像度
        topleft_x (float): 左上X座標
        topleft_y (float): 左上Y座標
        class_id (int): カテゴリ番号
        name (string): カテゴリ名
        c_color (AisvColor): AisvColorオブジェクト
    """
    _fields_ = [
        ("prob", c_float),
        ("width", c_float),
        ("height", c_float),
        ("topleft_x", c_float),
        ("topleft_y", c_float),
        ("class_id", c_int),
        ("name", c_char * 32),
        ("c_color", AisvColor)
    ]

class ObjectDetectionInfo(Structure):
    """ObjectDetectionInfoオブジェクト
    Attributes:
        result_list (ObjectDetectionItem): ObjectDetectionItemオブジェクト
        size (int): リストサイズ
    """
    _fields_ = [
        ("result_list", ObjectDetectionItem * 256),
        ("size", c_int)
    ]

class OrientedObjectDetectionItem(Structure):
    """OrientedObjectDetectionItemオブジェクト
    Attributes:
        prob (float): 可能性(1 = 100%, 0 = 0%)
        width (float): 横解像度
        height (float): 縦解像度
        center_x (float): 中心X座標
        center_y (float): 中心Y座標
        class_id (int): カテゴリ番号
        angle (int): 角度
        name (string): カテゴリ名
        c_color (AisvColor): AisvColorオブジェクト
    """
    _fields_ = [
        ("prob", c_float),
        ("width", c_float),
        ("height", c_float),
        ("center_x", c_float),
        ("center_y", c_float),
        ("class_id", c_int),
        ("angle", c_int),
        ("name", c_char * 32),
        ("c_color", AisvColor)
    ]

class OrientedObjectDetectionInfo(Structure):
    """OrientedObjectDetectionInfoオブジェクト
    Attributes:
        result_list (OrientedObjectDetectionItem): OrientedObjectDetectionItemオブジェクト
        size (int): リストサイズ
    """
    _fields_ = [
        ("result_list", OrientedObjectDetectionItem * 256),
        ("size", c_int)
    ]
    
class SegmentationInfo(Structure):
    """SegmentationInfoオブジェクト
    Attributes:
        mask (AisvImage): AisvImageオブジェクト
        category_size (int): リストサイズ
        category_list (AisvCategoryItem): AisvCategoryItemオブジェクト
    """
    _fields_ = [
        ("mask", AisvImage),
        ("category_size", c_int),
        ("category_list", AisvCategoryItem * 256)
    ]

class AisvPredictInfo(Structure):
    """AisvPredictInfoオブジェクト
    Attributes:
        predict_status (AisvStatus): AisvStatusオブジェクト
        segmentation_info (SegmentationInfo): SegmentationInfoオブジェクト
        objectdetection_info (ObjectDetectionInfo): ObjectDetectionInfoオブジェクト
        classification_info (ClassificationInfo): ClassificationInfoオブジェクト
        anomalydetection_info (AnomalyDetectionInfo): AnomalyDetectionInfoオブジェクト
        orientedobjectdetection_info (OrientedObjectDetectionInfo): OrientedObjectDetectionInfoオブジェクト
    """
    _fields_ = [
        ("predict_status", AisvStatus),
        ("segmentation_info", SegmentationInfo),
        ("objectdetection_info", ObjectDetectionInfo),
        ("classification_info", ClassificationInfo),
        ("anomalydetection_info", AnomalyDetectionInfo),
        ("orientedobjectdetection_info", OrientedObjectDetectionInfo)
    ]

# 関数のプロトタイプ定義
aisv_predictor.AisvPredictorCreate.restype = c_void_p
aisv_predictor.AisvPredictorCreate.argtypes = []

aisv_predictor.AisvPredictorCreateWithLogFolder.restype = c_void_p
aisv_predictor.AisvPredictorCreateWithLogFolder.argtypes = [c_char_p]

aisv_predictor.AisvPredictorDestroy.restype = c_bool
aisv_predictor.AisvPredictorDestroy.argtypes = [c_void_p]

aisv_predictor.AisvPredictorCheckSession.restype = c_bool
aisv_predictor.AisvPredictorCheckSession.argtypes = [c_void_p]

aisv_predictor.AisvPredictorCheckDongle.restype = c_bool
aisv_predictor.AisvPredictorCheckDongle.argtypes = [c_void_p]

aisv_predictor.AisvPredictorLoadModel.restype = c_bool
aisv_predictor.AisvPredictorLoadModel.argtypes = [c_void_p, c_char_p, c_bool, c_int]

aisv_predictor.AisvPredictorPredict.restype = c_bool
aisv_predictor.AisvPredictorPredict.argtypes = [c_void_p, POINTER(AisvImage), POINTER(AisvPredictInfo)]

aisv_predictor.AisvPredictorVisualizeResult.restype = AisvImage
aisv_predictor.AisvPredictorVisualizeResult.argtypes = [c_void_p, POINTER(AisvImage), POINTER(AisvPredictInfo), c_bool]

aisv_predictor.AisvPredictorGetModelInfo.restype = c_bool
aisv_predictor.AisvPredictorGetModelInfo.argtypes = [c_void_p, POINTER(AisvModelInfo)]

aisv_predictor.AisvPredictorGetCategoryInfo.restype = c_bool
aisv_predictor.AisvPredictorGetCategoryInfo.argtypes = [c_void_p, POINTER(AisvCategoryInfo)]

aisv_predictor.AisvPredictorReadImage.restype = AisvImage
aisv_predictor.AisvPredictorReadImage.argtypes = [c_void_p, c_char_p]

aisv_predictor.AisvPredictorSaveImage.restype = c_bool
aisv_predictor.AisvPredictorSaveImage.argtypes = [c_void_p, c_char_p, POINTER(AisvImage)]

aisv_predictor.AisvPredictorSaveLog.restype = None
aisv_predictor.AisvPredictorSaveLog.argtypes = [c_void_p, c_bool]

aisv_predictor.AisvPredictorSetCategoryThreshold.restype = c_bool
aisv_predictor.AisvPredictorSetCategoryThreshold.argtypes = [c_void_p, c_int, c_float]

aisv_predictor.AisvPredictorEnableAnoFastMode.restype = None
aisv_predictor.AisvPredictorEnableAnoFastMode.argtypes = [c_void_p, c_bool]

aisv_predictor.CreateEmptyImage.restype = AisvImage
aisv_predictor.CreateEmptyImage.argtypes = [c_int, c_int, c_int, ImageDepth]

aisv_predictor.FreeImage.restype = c_bool
aisv_predictor.FreeImage.argtypes = [POINTER(AisvImage)]

aisv_predictor.FreePredictInfo.restype = c_bool
aisv_predictor.FreePredictInfo.argtypes = [POINTER(AisvPredictInfo)]

# Pythonのラッパー関数
def create_predictor():
    """AisvPredictorオブジェクトの作成
    Returns:
        AisvPredictor : AisvPredictorオブジェクト
    """
    return aisv_predictor.AisvPredictorCreate()

def create_predictor_with_log_folder(log_folder):
    """AisvPredictorオブジェクトの作成
    Args:
        log_folder (string): ログファイルパス指定
    Returns:
        AisvPredictor : AisvPredictorオブジェクト
    """
    return aisv_predictor.AisvPredictorCreateWithLogFolder(log_folder.encode('utf-8'))

def destroy_predictor(predictor):
    """AisvPredictorオブジェクトの破棄
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorDestroy(predictor)

def check_session(predictor):
    """AisvPredictorオブジェクトの有効性チェック
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorCheckSession(predictor)

def check_dongle(predictor):
    """ライセンスドングルキーの有効性チェック
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorCheckDongle(predictor)

def load_model(predictor, model_path, use_gpu=True, gpu_index=0):
    """AIモデルファイルの読み込み
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        model_path (string): モデルファイルのパス
        use_gpu (bool, optional): GPU推論(Default=True)
        gpu_index (int, optional): GPUの指定(Default=0番)
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorLoadModel(predictor, model_path.encode('utf-8'), use_gpu, gpu_index)

def predict(predictor, aisv_img, predict_info):
    """推論実行
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        aisv_img (AisvImage): AisvImageバッファ
        predict_info (AisvPredictInfo): AisvPredictInfoオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorPredict(predictor, ctypes.byref(aisv_img), ctypes.byref(predict_info))

def visualize_result(predictor, aisv_img, predict_info, show_text=True, rgb2bgr=True):
    """推論結果画像の生成
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        aisv_img (AisvImage): AisvImageバッファ
        predict_info (AisvPredictInfo): AisvPredictInfoオブジェクト
        show_text (bool, optional): 結果文字列の表示(Default=True)
        rgb2bgr (bool): RGBをBGRの並びに変換(Default=True)
    Returns:
        ndarray : 画像データ(Numpy配列)
    """
    tempimg = aisv_predictor.AisvPredictorVisualizeResult(predictor, ctypes.byref(aisv_img), ctypes.byref(predict_info), show_text)
    tempbuf = ctypes.cast(tempimg.data, ctypes.POINTER(ctypes.c_ubyte * tempimg.width * tempimg.height * tempimg.channel))
    np_img = numpy.frombuffer(tempbuf.contents, dtype=numpy.uint8).copy().reshape(tempimg.height, tempimg.width, tempimg.channel)
    if rgb2bgr:
        np_img_bgr = np_img[:, :, ::-1]
        np_img = numpy.ascontiguousarray(np_img_bgr)
    free_image(tempimg)
    tempbuf = None
    del tempbuf
    return np_img

def get_model_info(predictor, model_info):
    """モデル情報の取得
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        model_info (AisvModelInfo): AisvModelInfoオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorGetModelInfo(predictor, ctypes.byref(model_info)) 

def get_category_info(predictor, category_info):
    """カテゴリ情報の取得
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        category_info (AisvCategoryInfo): AisvCategoryInfoオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorGetCategoryInfo(predictor, ctypes.byref(category_info)) 

def read_image(predictor, image_path):
    """画像ファイルの読み込み[RGBの色順データを生成]
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        image_path (string): 画像ファイルパス
    Returns:
        AisvImage : AisvImageバッファ
    """
    return aisv_predictor.AisvPredictorReadImage(predictor, image_path.encode('utf-8'))

def save_image(predictor, save_path, aisv_img):
    """画像ファイルの保存
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        save_path (string): 保存先パス
        aisv_img (AisvImage): AisvImageバッファ
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorSaveImage(predictor, save_path.encode('utf-8'), ctypes.byref(aisv_img))

def save_log(predictor, save_log=True):
    """ログ保存の有無設定
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        save_log (bool): 有効/無効(Default=True)
    """
    aisv_predictor.AisvPredictorSaveLog(predictor, save_log)

def set_category_threshold(predictor, index, threshold):
    """閾値設定の変更
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        index (int): カテゴリ番号
        threshold (float): 閾値
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.AisvPredictorSetCategoryThreshold(predictor, index, threshold)

def enable_ano_fast_mode(predictor, enable=True):
    """AnomalyDetection用、推論画像の生成の有無設定
    （Normalモードのモデルのみ利用可能）
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
        enable (bool): 有効/無効(Default=True)
    """
    aisv_predictor.AisvPredictorEnableAnoFastMode(predictor, enable)

def create_empty_image(width, height, channel, depth):
    """空のAisvImageバッファの作成[RGBの色順データを生成]
    Args:
        width (int): 横解像度
        height (int): 縦解像度
        channel (int): カラーチャンネル
        depth (ImageDepth): Bit深度

    Returns:
        AisvImage : AisvImageバッファ
    """
    return aisv_predictor.CreateEmptyImage(width, height, channel, depth)

def free_image(aisv_img):
    """AisvImageバッファの解放
    Args:
        aisv_img (AisvImage): AisvImageバッファ
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.FreeImage(ctypes.byref(aisv_img))

def free_predict_info(predict_info):
    """AisvPredictorオブジェクトの解放
    Args:
        predictor (AisvPredictor): AisvPredictorオブジェクト
    Returns:
        Bool: 実行結果
    """
    return aisv_predictor.FreePredictInfo(ctypes.byref(predict_info))

def create_image_from_numpy(np_img, bgr2rgb=True):
    """Numpy配列の画像データをAisvImage型へ変換[RGBの色順データが必要]
    Args:
        np_img (ndarray): Numpy配列の画像データ
        bgr2rgb (bool): BGRをRGBの並びに変換(Default=True)
    Raises:
        ValueError: 未サポートbit深度エラー
        RuntimeError: 変換エラー
    Returns:
        AisvImage : AisvImageバッファ
    """
    aisv_img = None
    ctypes_array = None
    if bgr2rgb:
        np_img_rgb = np_img[:, :, ::-1]
        np_img = numpy.ascontiguousarray(np_img_rgb)
    if np_img.dtype == "uint8":
        aisv_img = aisv_predictor.CreateEmptyImage(np_img.shape[1], np_img.shape[0], np_img.ndim, ImageDepth.IMAGE_DEPTH_UINT8)
        ctypes_array = (ctypes.c_uint8 * np_img.size).from_buffer(np_img)
    elif np_img.dtype == "uint16":
        aisv_img = aisv_predictor.CreateEmptyImage(np_img.shape[1], np_img.shape[0], np_img.ndim, ImageDepth.IMAGE_DEPTH_UINT16)
        ctypes_array = (ctypes.c_uint16 * np_img.size).from_buffer(np_img)
    else:
        raise ValueError(f"Unsupported data type: {np_img.dtype}. Please use uint8 or uint16.")
    if aisv_img is not None and ctypes_array is not None:
        ctypes.memmove(aisv_img.data, ctypes.addressof(ctypes_array), np_img.nbytes)
    else:
        raise RuntimeError("Failed to create AisvImage or ctypes array.")
    ctypes_array = None
    del ctypes_array
    return aisv_img

def create_numpy_from_image(aisv_img, rgb2bgr=True):
    """AisvImage配列の画像データ[RGBの色順データ]をNumpy型へ変換
    Args:
        aisv_img (AisvImage): AisvImageバッファ
        rgb2bgr (bool): RGBをBGRの並びに変換(Default=True)
    Raises:
        ValueError: 未サポートbit深度エラー
        RuntimeError: 変換エラー
    Returns:
        ndarray : 画像データ(Numpy配列)
    """
    if aisv_img.depth.value == ImageDepth.IMAGE_DEPTH_UINT8:
        tempbuf = ctypes.cast(aisv_img.data, ctypes.POINTER(ctypes.c_ubyte * aisv_img.width * aisv_img.height * aisv_img.channel))
        np_img = numpy.frombuffer(tempbuf.contents, dtype=numpy.uint8).copy().reshape(aisv_img.height, aisv_img.width, aisv_img.channel)
    elif aisv_img.depth.value == ImageDepth.IMAGE_DEPTH_UINT16:
        tempbuf = ctypes.cast(aisv_img.data, ctypes.POINTER(ctypes.c_uint16 * aisv_img.width * aisv_img.height * aisv_img.channel))
        np_img = numpy.frombuffer(tempbuf.contents, dtype=numpy.uint16).copy().reshape(aisv_img.height, aisv_img.width, aisv_img.channel)
    else:
        raise ValueError(f"Unsupported data type: {get_imagedepth_name(aisv_img.depth)}. Please use uint8 or uint16.")
    if rgb2bgr:
        np_img_bgr = np_img[:, :, ::-1]
        np_img = numpy.ascontiguousarray(np_img_bgr)
    tempbuf = None
    del tempbuf
    return np_img

def get_modeltype_name(model_type):
    """ModelType型の名前取得
    Args:
        model_type (ModelType): ModelTypeオブジェクト
    Returns:
        string: 定義名
    """
    return ModelType.name.get(model_type.value)
    
def get_modetype_name(mode_type):
    """ModeType型の名前取得
    Args:
        mode_type (ModeType): ModeTypeオブジェクト

    Returns:
        string: 定義名
    """
    return ModeType.name.get(mode_type.value)
    
def get_imagedepth_name(image_depth):
    """ImageDepth型の名前取得
    Args:
        image_depth (ImageDepth): ImageDepthオブジェクト

    Returns:
        string: 定義名
    """
    return ImageDepth.name.get(image_depth.value)

def get_aisvstatus_name(aisv_status):
    """AisvStatus型の名前取得
    Args:
        aisv_status (AisvStatus): AisvStatusオブジェクト
    Returns:
        string: 定義名
    """
    return AisvStatus.name.get(aisv_status.value)
