import os
import sys
import cv2
import numpy
import bin.aisvino_lib as ais

# モデルのファイルパス指定
modelpath = os.path.join(os.path.dirname(__file__)) + "/example_data/AnomalyDetection/IR_model/AnomalyDetection.ditir"
# 推論画像のファイルパス指定
imgpath = os.path.join(os.path.dirname(__file__)) + "/example_data/AnomalyDetection/image/NG/IMG_073.png"
# 推論モジュールの初期化
predictor = ais.create_predictor()

# ログ保存無効
ais.save_log(predictor, False)
# USBドングルチェック
if ais.check_dongle(predictor) != True :
    print("CheckDongle: Error\n")
    sys.exit(0)
else :
    print(f"CheckDongle: the USB dongle key is Connected.\n")

# デバイスチェック
deviceinfo = ais.DeviceInfo()
ais.get_devices(predictor, deviceinfo)
if deviceinfo.size > 0 :
    for i in range(deviceinfo.size):
        print(f"index: {deviceinfo.device_list[i].id}, DeviceName: {deviceinfo.device_list[i].name.decode()}, AISVinoSupport: {str(deviceinfo.device_list[i].supported)}")
else :
    print(f"no Devices for AISVino.")

# デバイス選択
index = int(input('Input the target device index number:'))
if deviceinfo.device_list[index].supported != True :
    print("Please select the supported device\n")
    sys.exit(0)

# モデルの読み込み
ais.load_model(predictor, modelpath, [index], 1)
# モデル読み込み完了のチェック
if ais.check_session(predictor) != True :
    print("CheckSession: Error\n")
    sys.exit(0)
else :
    print(f"CheckSession: the model is loaded.\n")

# 各種情報オブジェクトの作成
modelinfo = ais.AisvModelInfo()
categoryinfo = ais.AisvCategoryInfo()
predictinfo = ais.AisvPredictInfo()

# モデル情報の取得
res = ais.get_model_info(predictor, modelinfo)
print(f"Get ModelInfo: {str(res)}, status: {ais.get_aisvstatus_name(modelinfo.model_status)}")
modeltype = ais.get_modeltype_name(modelinfo.model_type)
modetype = ais.get_modetype_name(modelinfo.mode_type)
print(f"ProjectName: {modelinfo.project_name.decode()}, TaskName: {modelinfo.task_name.decode()}\nModelType: {modeltype}, ModeType: {modetype} \n")

# カテゴリ情報の取得
res = ais.get_category_info(predictor, categoryinfo)
if categoryinfo.size > 0 :
    for i in range(categoryinfo.size) :
        print(f"index: {categoryinfo.category_list[i].index}, CategoryName: {str(categoryinfo.category_list[i].name.decode())}, Threshold: {categoryinfo.category_list[i].threshold}")
print("\n")

#FastAnomalyModeを有効化するとマスク画像を作成しない代わりに高速推論になる
#"anomalydetection_info.mask"や"ais.visualize_result()"は真っ黒の画像を返す
if modeltype == "ANOMALYDETECTION" :
    #ais.enable_ano_fast_mode(predictor,True)
    ais.enable_ano_fast_mode(predictor,False)
print("\n")

#プレビューウィンドウの準備
cv_windowname = "Preview: Press ESC key for exit"
cv2.namedWindow(cv_windowname, cv2.WINDOW_AUTOSIZE)
cv_key = 0

# 結果表示
# ESCキーを押すと停止
while cv_key != 27 :
    # ウィンドウを閉じたら終了
    if cv2.getWindowProperty(cv_windowname, cv2.WND_PROP_VISIBLE) == 0 :
        break
    
    # ファイルから推論用AISVImage型画像バッファの作成：(RGBの並び)
    r_aisv_img = ais.read_image(predictor, imgpath)
    print(f"AISVImageInfo from file: {str(r_aisv_img.width)} x {str(r_aisv_img.height)} [{ais.get_imagedepth_name(r_aisv_img.depth)} : {str(r_aisv_img.channel)}channel]") 
    
    # 推論
    res = ais.predict(predictor, r_aisv_img, predictinfo)
    print(f"Predict from read_image: {str(res)}, status: {ais.get_aisvstatus_name(predictinfo.predict_status)}")
    if modeltype == "ANOMALYDETECTION" :
        print(f"Result: {predictinfo.anomalydetection_info.name.decode()}, Score: {predictinfo.anomalydetection_info.score}\n")
    # Nummpy型推論結果画像の作成：(BGRの並び)
    r_np_img = ais.visualize_result(predictor, r_aisv_img, predictinfo)
    # AISVisionでAisvImageバッファを静止画保存
    ais.save_image(predictor, "r_aisv_img.jpg", r_aisv_img)
    # リソースの解放
    ais.free_image(r_aisv_img)
    ais.free_predict_info(predictinfo)
    
    # Nummpy型のデータをコピー
    temp_np_img = numpy.copy(r_np_img)
    temp_np_img = cv2.putText(temp_np_img, 'create from numpy data', (10, r_np_img.shape[0]-50), cv2.FONT_HERSHEY_SIMPLEX, 1.0, (255, 128, 0), thickness=2)
    # Numpy型から推論用AISVImage型画像バッファの作成：(RGBの並び)
    u_aisv_img = ais.create_image_from_numpy(temp_np_img)
    print(f"AISVImageInfo from numpy: {str(u_aisv_img.width)} x {str(u_aisv_img.height)} [{ais.get_imagedepth_name(r_aisv_img.depth)} : {str(u_aisv_img.channel)}channel]") 
    
    # 推論
    res = ais.predict(predictor, u_aisv_img, predictinfo)
    print(f"Predict from numpy: {str(res)}, status: {ais.get_aisvstatus_name(predictinfo.predict_status)}")
    if modeltype == "ANOMALYDETECTION" :
        print(f"Result: {predictinfo.anomalydetection_info.name.decode()}, Score: {predictinfo.anomalydetection_info.score}\n")
    # Nummpy型推論結果画像の作成：(BGRの並び)
    u_np_img = ais.visualize_result(predictor, u_aisv_img, predictinfo)
    # AISVisionでAisvImageバッファを静止画保存
    ais.save_image(predictor, "u_aisv_img.jpg", u_aisv_img)
    # リソースの解放
    ais.free_image(u_aisv_img)
    ais.free_predict_info(predictinfo)
    
    # OpenCVでMat型のデータを静止画保存
    cv2.imwrite("r_cv_img.jpg", r_np_img)
    cv2.imwrite("u_cv_img.jpg", u_np_img)
    
    # 各推論結果の表示
    cv2.imshow(cv_windowname, cv2.hconcat([r_np_img, u_np_img]))
    cv_key = cv2.waitKey(1)
    pass

# モデルの解放
ais.destroy_predictor(predictor)

# OpenCVプレビュー画面破棄
cv2.destroyAllWindows()

print(":example end:")
sys.exit(0)
